/*
  USB ByteBlaster
SM -   :
1.    FT245;
2.     :
   2.1     JTAG
   2.2    - "shiftout.v"
   2.3    - bytecounter
3.       Echo   
   3.1    CDONE, DATAOUT
   3.2    
*/
`include "shiftout.v"

module JTAGcore  (GREEN_LED, RED_LED, 			
				  N_RD, WR,		
                  JTCK, N_CONF, N_CS, N_CE, ASDI,  
 				  CDONE, DATAOUT,
	              d[7:0], CLK,
                  N_RXF, N_TXE,  RESET);
  input RESET;	       // 
  input N_TXE, N_RXF;  // FT245  / 
  input CLK;           //  EPM3064

  inout [7:0]d;        //    EPM3064  FT245

  output GREEN_LED, RED_LED;                // 

// JTAG  
  input CDONE, DATAOUT;
  output JTCK, N_CONF, N_CS, N_CE, ASDI;    //  JTAG/Config
  
  output N_RD, WR;     //  FT245

//       
//   
  reg N_CE;     // N_CE == [2]  -
  reg N_CS;     // N_CS == [3]  -  
  reg R_JTCK;   // TCK == d[0]  -   
  reg R_ASDI;   // TDI == d[4]  -   
  reg N_CONF;   // TMS == d[1]  -  

  wire[7:0] To_FT245;	//      FT245

  reg RED_LED; // JTAG / 
   
//        
  wire RXF, TXE;	
  reg RD,WR;
  reg echo;

  reg[3:0] SM;	// State Machine    FT245
  parameter Idle=0, 
            Parcer=1, 
            ReadString  =2, 
			ReadString2 =3, 
            ReadString3 =4, 

//    FT245 			
			WaitParcer = 10,
			wait_WR_Active=11,
			wait_RD_Active=12,
			
            RD_Active=13, 
            WR_Active=14, 
            WR_End=15 ;

//  
  reg [5:0] ByteCounter;

//   
  wire[7:0] dout; 	//   
  wire so;			//   (TDI)
  wire rdy;         //    
  wire T_CLK;		//   (TCK)
  reg ByteLoad;     //    

//  
  shiftout shifter(dout, so, rdy, T_CLK, d[7:0], CDONE, ByteLoad, RESET, CLK); 

//        
//   
  reg direct;	

assign GREEN_LED  = ~ echo;

assign RXF = ~ N_RXF;	//     
assign TXE = ~ N_TXE;	//      
assign N_RD = ~ RD;		//      	

//     FT245    WR
assign d = WR ? To_FT245 : 8'bZ ;

// JTAG
//      JTAG
//   .  TCK  TDI
assign JTCK = direct ? R_JTCK : T_CLK;
assign ASDI = direct ? R_ASDI : so;

//      JTAG
assign To_FT245[7:0] = direct ? {dout[7:2],DATAOUT,CDONE}: dout[7:0];

//   
//  -  /   FT245
//  -  / JTAG   FT245
always@(posedge CLK or posedge RESET)
begin
 if (RESET) 
  begin
   SM=Idle;
   RD=0;
   WR=0;	
   echo=0;
   direct=1;
//  
   ByteCounter=0;
   ByteLoad=0;
//  
   RED_LED=1;	// 
//  JTAG
   N_CONF=0;   // TMS == d[1]  -  
   N_CE=1;     // N_CE == [2]  -
   N_CS=1;     // N_CS == [3]  -  

   R_JTCK=0;   // TCK == d[0]  -   
   R_ASDI=0;   // TDI == d[4]  -   
  end
  else

 case(SM)
 Idle:  		//    / 
  begin
    if (RXF)
     begin
      SM=WaitParcer;     //  
	  RD=1;          //    FT245   
     end
    end 	

WaitParcer:
 begin
  SM=Parcer;
 end
  
	
 Parcer:  //     
     begin
       echo=d[6]; // c      
      if (d[7]==1)          //  
       begin
        ByteCounter=d[5:0]; //    
        direct=0;           //    
        RD=0;               //    FT245
        SM=ReadString;		//   
       end   
      else if (d[5]==1) //  
       begin
        direct=1;
	    SM=RD_Active;     //     
	   end 
	  else if (d[4]==1)   //    Z  JTAG?
	   begin
	    if (d[3:0]==4'b1111) RED_LED=1;
	     else if (d[3:0]==4'b1110) RED_LED=0;
        RD=0; 
	    SM=Idle;   //  echo   
	   end
  end

 RD_Active:
 begin
	  R_JTCK   =d[0]; // TCK
      N_CONF   =d[1]; // TMS
	  N_CE     =d[2]; // ce
      N_CS     =d[3]; // cs 
      R_ASDI   =d[4]; // TDI
      RD=0; 
      SM=wait_RD_Active;
 end 

wait_RD_Active:
 begin
  if (echo) SM=WR_Active;  //     
   else SM=Idle;          //  echo   
 end

 WR_Active:
   begin
	  if (TXE)
	   begin
        WR=1;
        SM=wait_WR_Active;
       end
 end

wait_WR_Active:
begin
 SM=WR_End;
end

 WR_End:
      begin
       WR=0;
	if (ByteCounter==4'b000)  SM=Idle;
	 else SM=ReadString;
 end

 ReadString:
     begin
     if (ByteCounter==4'b0000)  //  
      SM=Idle; 
     else if (RXF==1)  //    
      begin
       RD=1;
       SM=ReadString2;
 	  end    
 end

 ReadString2:
      begin
       ByteLoad=1;  //     shifter 
       if (rdy==0)  //   
         begin
          ByteLoad=0; //   
		  RD=0;	      //   FT245 
          SM=ReadString3;
         end
 end 

 ReadString3:
      begin
	   if (rdy && ~echo) //  ,    
	    begin
	     ByteCounter=ByteCounter-1;
	     SM=ReadString;  //    
	    end
	   else if ((rdy) && (echo)) //  ,   
	    begin
	     ByteCounter=ByteCounter-1;
         SM=WR_Active;
        end
 end
     default:
      begin
      SM=Idle;
      WR=0;
      RD=0;
      echo=0;
	  direct=1;	
     end
   endcase 
end //always@(posedge CLK or posedge RESET)
endmodule

